import { basicSetup, EditorState, EditorView } from "codemirror";
import { ssrSync } from "#ssr";
// @ts-ignore
import type { ScriptPayload } from "../views/editor.marko";
// @ts-ignore
import QuestionRender from "@/q+a/components/Question.marko";

declare const payload: ScriptPayload;
const date = new Date(payload.date);

const main = document.getElementById("edit-grid")! as HTMLDivElement;
const preview = document.getElementById("preview")! as HTMLDivElement;

function updatePreview(text: string) {
  preview.innerHTML = ssrSync(
    <QuestionRender
      question={{
        id: payload.id,
        qmid: payload.qmid,
        text: text,
        type: payload.type,
        date,
      }}
    />,
  ).text;
}

const startState = EditorState.create({
  doc: payload.text,
  extensions: [
    basicSetup,
    EditorView.darkTheme.of(true),
    EditorView.updateListener.of((update) => {
      if (update.docChanged) {
        updatePreview(update.state.doc.toString());
      }
    }),
    EditorView.lineWrapping,
  ],
  // selection: EditorSelection.create([
  //   EditorSelection.cursor(0),
  // ], 0),
});

const view = new EditorView({
  state: startState,
  parent: document.getElementById("editor")!,
});
view.focus();

(globalThis as any).onCommitQuestion = wrapAction(async () => {
  const text = view.state.doc.toString();
  const res = await fetch(`/admin/q+a/${payload.id}`, {
    method: "PATCH",
    headers: {
      Accept: "application/json",
      "Content-Type": "application/json",
    },
    body: JSON.stringify({
      text,
      type: payload.type,
    }),
  });
  if (!res.ok) {
    throw new Error("Failed to update question, status: " + res.status);
  }
  if (location.search.includes("return=inbox")) {
    location.href = "/admin/q+a/inbox";
  } else {
    location.href = "/q+a#q" + payload.id;
  }
});
(globalThis as any).onDelete = wrapAction(async () => {
  if (confirm("Are you sure you want to delete this question?")) {
    const res = await fetch(`/admin/q+a/${payload.id}`, {
      method: "DELETE",
      headers: {
        Accept: "application/json",
      },
    });
    if (!res.ok) {
      throw new Error("Failed to delete question, status: " + res.status);
    }
    location.href = document.referrer || "/admin/q+a";
  }
});
(globalThis as any).onTypeChange = () => {
  payload.type = parseInt(
    (document.getElementById("type") as HTMLSelectElement).value,
  );
  updatePreview(view.state.doc.toString());
};

function wrapAction(cb: () => Promise<void>) {
  return async () => {
    main.style.opacity = "0.5";
    main.style.pointerEvents = "none";
    const inputs = main.querySelectorAll("button,select,input") as NodeListOf<
      HTMLButtonElement
    >;
    inputs.forEach((b) => {
      b.disabled = true;
    });
    try {
      await cb();
    } catch (e: any) {
      main.style.opacity = "1";
      main.style.pointerEvents = "auto";
      inputs.forEach((b) => {
        b.disabled = false;
      });
      alert(e.message);
    }
  };
}
