async function qaDate(el: HTMLAnchorElement) {
  if (el.hasAttribute("data-copy-state")) return;
  const time = el.children[0] as HTMLTimeElement;
  const prevText = time.textContent;
  const url = `${window.location.origin}/q+a/${
    time.dateTime.replace(/[^\d]/g, "").slice(2, 12)
  }`;
  el.setAttribute("data-copy-state", "1");
  try {
    await navigator.clipboard.writeText(url);
    time.textContent = "copied permalink to question";
    time.classList.add("winner");
  } catch (error) {
    time.textContent = "failed to copy :(";
  }
  setTimeout(() => {
    el.removeAttribute("data-copy-state");
    time.textContent = prevText;
    time.classList.remove("winner");
  }, 1500);
}
requestAnimationFrame(() => {
  // Make every link clickable
  document.querySelectorAll("e-").forEach((questionRoot) =>
    (questionRoot.querySelector("a:not([href])") as HTMLAnchorElement).href =
      "#"
  );
  // Avoid attaching 1500+ event listeners
  document.querySelector("main")!.addEventListener(
    "click",
    (event, element = event.target as HTMLAnchorElement) => {
      if (
        !(event.button ||
          event.which != 1 ||
          event.metaKey ||
          event.ctrlKey ||
          event.shiftKey ||
          event.altKey ||
          event.defaultPrevented)
      ) {
        while (element && element !== document.body) {
          if (
            element.nodeName.toUpperCase() === "A" &&
            element.getAttribute("href") === "#" &&
            element.parentElement?.tagName === "E-"
          ) {
            event.preventDefault();
            qaDate(element);
            return;
          }
          element =
            (element.assignedSlot ?? element.parentNode) as HTMLAnchorElement;
        }
      }
    },
  );
});
